@@ -561,15 +561,13 @@ def test_plotly(pyi_builder):
 def test_dash(pyi_builder):
     pyi_builder.test_source("""
         import dash
-        import dash_core_components as dcc
-        import dash_html_components as html
         from dash.dependencies import Input, Output
 
         app = dash.Dash(__name__)
-        app.layout = html.Div(
+        app.layout = dash.html.Div(
             [
-                dcc.Input(id='input_text', type='text', placeholder='input type text'),
-                html.Div(id='out-all-types'),
+                dash.dcc.Input(id='input_text', type='text', placeholder='input type text'),
+                dash.html.Div(id='out-all-types'),
             ]
         )
 
@@ -582,14 +580,13 @@ def test_dash(pyi_builder):
         """)
 
 
-@importorskip('dash_table')
+@importorskip('dash')
 def test_dash_table(pyi_builder):
     pyi_builder.test_source("""
         import dash
-        import dash_table
 
         app = dash.Dash(__name__)
-        app.layout = dash_table.DataTable(
+        app.layout = dash.dash_table.DataTable(
             id='table',
             columns=[{'name': 'a', 'id': 'a'}, {'name': 'b', 'id': 'b'}],
             data=[{'a': 1, 'b': 2}, {'a': 3, 'b': 4}],
@@ -597,15 +594,15 @@ def test_dash_table(pyi_builder):
         """)
 
 
+@importorskip('dash')
 @importorskip('dash_bootstrap_components')
 def test_dash_bootstrap_components(pyi_builder):
     pyi_builder.test_source("""
         import dash
         import dash_bootstrap_components as dbc
-        import dash_html_components as html
 
         app = dash.Dash(external_stylesheets=[dbc.themes.BOOTSTRAP])
-        alert = dbc.Alert([html.H4('Well done!', className='alert-heading')])
+        alert = dbc.Alert([dash.html.H4('Well done!', className='alert-heading')])
         """)
 
 
@@ -630,9 +627,13 @@ def test_office365(pyi_builder):
     pyi_builder.test_source("""
         from office365.runtime.auth.providers.saml_token_provider import SamlTokenProvider
 
-        SamlTokenProvider._prepare_request_from_template('FederatedSAML.xml', {})
-        SamlTokenProvider._prepare_request_from_template('RST2.xml', {})
-        SamlTokenProvider._prepare_request_from_template('SAML.xml', {})
+        provider = SamlTokenProvider("https://example.com", "bob", "bob's password", "")
+        parameters = dict.fromkeys(["auth_url", "message_id", "username", "password", "created", "expires",
+                                    "issuer", "serviceTokenUrl", "assertion_node"], "")
+
+        provider._prepare_request_from_template("FederatedSAML.xml", parameters)
+        provider._prepare_request_from_template("RST2.xml", parameters)
+        provider._prepare_request_from_template("SAML.xml", parameters)
         """)
 
 
@@ -894,6 +895,13 @@ def test_adbutils(pyi_builder):
             """)
 
 
+@importorskip("apkutils")
+def test_apkutils(pyi_builder):
+    pyi_builder.test_source("""
+        from apkutils import APK
+    """)
+
+
 @importorskip("pymediainfo")
 def test_pymediainfo(pyi_builder):
     pyi_builder.test_source("""
@@ -976,6 +984,17 @@ def test_panel(pyi_builder):
         """)
 
 
+@importorskip("pandas_flavor")
+def test_pandas_flavor(pyi_builder):
+    pyi_builder.test_source("""
+        from pandas_flavor import register_dataframe_accessor
+
+        @register_dataframe_accessor("dummy")
+        class DummyAccessor:
+            pass
+    """)
+
+
 @importorskip("pyviz_comms")
 def test_pyviz_comms(pyi_builder):
     pyi_builder.test_source("""
@@ -2663,3 +2682,199 @@ def test_niquests(pyi_builder):
         except (niquests.ConnectionError, niquests.Timeout):
             pass
     """)
+
+
+@importorskip('emoji')
+def test_emoji(pyi_builder):
+    pyi_builder.test_source("""
+        import emoji
+    """)
+
+
+# Basic test for urllib3 - either from urllib3 or urllib3-future
+@importorskip('urllib3')
+def test_urllib3(pyi_builder):
+    pyi_builder.test_source("""
+        import urllib3
+
+        http = urllib3.PoolManager()
+        try:
+            resp = http.request("GET", "https://localhost/robots.txt")
+        except urllib3.exceptions.HTTPError:
+            pass
+    """)
+
+
+# Basic test for urllib3_future from urllib3-future
+@importorskip('urllib3_future')
+def test_urllib3_future(pyi_builder):
+    pyi_builder.test_source("""
+        import urllib3_future
+
+        http = urllib3_future.PoolManager()
+        try:
+            resp = http.request("GET", "https://localhost/robots.txt")
+        except urllib3_future.exceptions.HTTPError:
+            pass
+    """)
+
+
+@importorskip('black')
+def test_black(pyi_builder):
+    pyi_builder.test_source("""
+        import black
+
+        mode = black.Mode(
+            target_versions=set(),  # auto-detect
+            line_length=120,
+        )
+
+        code = "print ('hello, world') "
+        print("Original code: {code!r}")
+
+        reformatted_code = black.format_file_contents(
+            code,
+            fast=False,
+            mode=mode,
+        )
+        print("Reformatted code: {code!r}")
+
+        # Try reformatting again - this should raise black.NothingChanged
+        try:
+            reformatted_code2 = black.format_file_contents(
+                reformatted_code,
+                fast=False,
+                mode=mode,
+            )
+        except black.NothingChanged:
+            pass
+        else:
+            raise RuntimeError("black.NothingChanged exception was not raised!")
+    """)
+
+
+@importorskip('blib2to3')
+def test_black_blib2to3_pygram(pyi_builder):
+    pyi_builder.test_source("""
+        import blib2to3.pygram
+    """)
+
+
+@importorskip('blib2to3')
+def test_black_blib2to3_pytree(pyi_builder):
+    pyi_builder.test_source("""
+        import blib2to3.pytree
+    """)
+
+
+@importorskip('frictionless')
+def test_frictionless(pyi_builder, tmp_path):
+    # Example CSV file, taken from upstream example at
+    # https://framework.frictionlessdata.io/docs/getting-started.html
+    csv_file = tmp_path / "example.csv"
+    csv_file.write_text("\n".join([
+        "id,name,,name"
+        "1,english"
+        "1,english"
+        ""
+        "2,german,1,2,3"
+    ]))
+
+    pyi_builder.test_source("""
+        import sys
+        import pprint
+
+        import frictionless
+
+        filename = sys.argv[1]
+
+        # frictionless.describe()
+        print("Testing frictionless.describe...")
+        output = frictionless.describe(filename)
+        pprint.pprint(output)
+
+        # frictionless.extract()
+        print("Testing frictionless.extract...")
+        output = frictionless.extract(filename)
+        pprint.pprint(output)
+
+        # frictionless.validate()
+        print("Testing frictionless.validate...")
+        output = frictionless.validate(filename)
+        pprint.pprint(output)
+    """, app_args=[str(csv_file)])
+
+
+@importorskip('pandas')
+@importorskip('pandera')
+def test_pandera(pyi_builder):
+    # Example from pandera's Quick Start
+    pyi_builder.test_source("""
+        import pandas as pd
+        import pandera as pa
+
+        # data to validate
+        df = pd.DataFrame({
+            "column1": [1, 4, 0, 10, 9],
+            "column2": [-1.3, -1.4, -2.9, -10.1, -20.4],
+            "column3": ["value_1", "value_2", "value_3", "value_2", "value_1"]
+        })
+
+        # define schema
+        schema = pa.DataFrameSchema({
+            "column1": pa.Column(int, checks=pa.Check.le(10)),
+            "column2": pa.Column(float, checks=pa.Check.lt(-1.2)),
+            "column3": pa.Column(str, checks=[
+                pa.Check.str_startswith("value_"),
+                # define custom checks as functions that take a series as input and
+                # outputs a boolean or boolean Series
+                pa.Check(lambda s: s.str.split("_", expand=True).shape[1] == 2)
+            ]),
+        })
+
+        validated_df = schema(df)
+        print(validated_df)
+    """)
+
+
+@importorskip('tkinterweb')
+def test_tkinterweb(pyi_builder):
+    pyi_builder.test_source("""
+        import tkinter
+        import tkinterweb
+
+        root = tkinter.Tk()
+        frame = tkinterweb.HtmlFrame(root, messages_enabled=False)
+
+        # Load a test string that uses all files that should have been bundled
+        frame.load_html(
+                            "<img src='this path doesn't exist' alt='Hello, World!'/> \
+                            <p>Hello Again!</p> \
+                            <select><option>Hi...</option></select>"
+                            )
+    """)
+
+
+@importorskip('tkinterweb_tkhtml')
+def test_tkinterweb_tkhtml(pyi_builder):
+    pyi_builder.test_source("""
+        import tkinter
+        import tkinterweb_tkhtml
+
+        root = tkinter.Tk()
+
+        folder = tkinterweb_tkhtml.get_tkhtml_folder()
+        tkinterweb_tkhtml.load_tkhtml(root, folder)
+
+        frame = tkinter.Widget(root, "html")
+
+        # Load a test string
+        frame.tk.call(frame._w, "parse", "<p>Hello, World!</p>")
+    """)
+
+
+@importorskip('narwhals')
+def test_narwhals(pyi_builder):
+    pyi_builder.test_source("""
+        import narwhals
+    """)
